/* 
 * File:   CDBResponseHandler.cpp
 * Author: martynia
 * 
 * Created on August 9, 2016, 2:21 PM
 */

#include "base/CDBResponseHandler.h"
#include "base/CDBException.h"
#include "base/CDBPermanentException.h"
#include "base/CDBTemporaryException.h"

namespace MAUS {
    namespace CDB {

        CDBResponseHandler::CDBResponseHandler() {
            doc = NULL;
        }

        CDBResponseHandler::~CDBResponseHandler() {
            xmlFreeDoc(doc);
            xmlCleanupParser();
            std::cout << "XML parser cleared " << std::endl;
        }

        void CDBResponseHandler::analyze(const char* xml) {

            LIBXML_TEST_VERSION

                    // xmlParserOption myoption = XML_PARSE_NOERROR;
                    int options = (int) XML_PARSE_NOERROR;
            doc = xmlReadDoc((const unsigned char*) xml, NULL, NULL, options); // read from a in-memory string 

            if (NULL == doc) return; // this is a 'normal' route, for a binary decoded output

            cur = xmlDocGetRootElement(doc);

            if (cur == NULL) {
                std::cerr << " CDB response error handler: No XML root element, no message stored (non fatal but unexpected)" << std::endl;
                message.assign(" CDB response error handler: No XML root element, no message stored (non fatal but unexpected)");
                throw (CDBException(message));
            }// look only for error and warning tags

            else if (0 == xmlStrcmp(cur->name, (const xmlChar *) "error") ||
                    0 == xmlStrcmp(cur->name, (const xmlChar *) "warning")||
                    0 == xmlStrcmp(cur->name, (const xmlChar *) "ok")) {

                bool error = (0 == xmlStrcmp(cur->name, (const xmlChar *) "error"));
                bool warning = (0 == xmlStrcmp(cur->name, (const xmlChar *) "warning"));

                // the actual message is here:
                cur = cur->children;
                if (NULL == cur) {
                    std::cerr << " CDB response error handler: No child node within <ok> <error> or <warning> tag, no message stored (non fatal)" << std::endl;
                    message.assign(" CDB response error handler: No child node within <ok> <error> or <warning> tag, no message stored (non fatal)");
                } else if (xmlStrcmp(cur->name, (const xmlChar *) "text")) {
                    // this should be a text node (cur->name is "text")
                    std::cerr << " CDB response error handler: No text node within <ok> <error> or <warning> tag, no message stored (non fatal) " << std::endl;
                    message.assign(" CDB response error handler: No text node within <ok> <error> or <warning> tag, no message stored (non fatal) ");
                } else {
                    message.assign((const char *) cur->content);
                }
                if (error) {
                    throw (CDBPermanentException(message));
                } else if (warning){
                    throw (CDBTemporaryException(message));
                } // must be <ok> tag now
            }
            // do nothing for other XML tags
        }
        const std::string & CDBResponseHandler::getMessage() const {
            return this->message;
        }
    }
}