/* 
 * File:   calibration.cpp
 * Author: martynia
 * 
 * Created on August 15, 2016, 12:08 PM
 */


#include <soapCalibrationImplPortBindingProxy.h>
#include <soapH.h>
#include <string>
#include <sstream>
#include <base/CDBException.h>
#include <base/CDBPermanentException.h>
#include <base/CDBResponseHandler.h>

#include "calibration/calibration.h"

namespace MAUS {
    namespace CDB {

        void Calibration::getDetectorCalibrationForRun(const char* device, const char* type, int runNo, std::string& result) {

            CDBResponseHandler handler;

            ns2__getDetectorCalibrationForRun request;
            ns2__getDetectorCalibrationForRunResponse response;

            // arg0/1/2 are pointers to std::string

            std::string sdevice = std::string(device);
            request.arg0 = &sdevice;

            std::string stype = std::string(type);
            request.arg1 = &stype;

            std::string srun = std::to_string(runNo); // C++11
            request.arg2 = &srun;

            const char *soap_action = NULL;

            /* invoke the web service operation */
            if (proxy.getDetectorCalibrationForRun(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getDetectorCalibrationForRun: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());
            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;
        }

        void Calibration::getDetectorCalibrationForDate(const char* device, const char* type, const char* timestamp, std::string& result) {
            CDBResponseHandler handler;

            ns2__getDetectorCalibrationForDate request;
            ns2__getDetectorCalibrationForDateResponse response;

            const char *soap_action = NULL;

            std::string sdevice = std::string(device);
            request.arg0 = &sdevice;

            std::string stype = std::string(type);
            request.arg1 = &stype;

            std::string stimestp = std::string(timestamp); // C++11
            request.arg2 = &stimestp;

            /* invoke the web service operation */
            if (proxy.getDetectorCalibrationForDate(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getDetectorCalibrationForDate: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());
            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;
        }

        void Calibration::getDetectorCalibrationForId(int id, std::string& result) {
            CDBResponseHandler handler;

            ns2__getDetectorCalibrationForId request;
            ns2__getDetectorCalibrationForIdResponse response;

            // arg0/1/2 are pointers to std::string

            std::string sid = std::to_string(id); // C++11
            request.arg0 = &sid;

            const char *soap_action = NULL;

            /* invoke the web service operation */
            if (proxy.getDetectorCalibrationForId(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getDetectorCalibrationForId: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());
            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;

        }

        void Calibration::getCurrentDetectorCalibration(char* device, char* type, std::string& result) {
            CDBResponseHandler handler;

            ns2__getCurrentDetectorCalibration request;
            ns2__getCurrentDetectorCalibrationResponse response;

            const char *soap_action = NULL;

            std::string sdevice = std::string(device);
            request.arg0 = &sdevice;

            std::string stype = std::string(type);
            request.arg1 = &stype;

            /* invoke the web service operation */
            if (proxy.getCurrentDetectorCalibration(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_->__ptr) {
                    result.assign((char*) response.return_->__ptr, response.return_->__size);
                } else {
                    std::string nullResp("getCurrentDetectorCalibration: CDB server returned no data (response buffer is NULL)");
                    throw CDBPermanentException(nullResp);
                }
                // now check if response is not simply an error or warning XML document
                handler.analyze(result.c_str());
            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;
        }

        void Calibration::getStatus(std::string& result) {

            ns2__getStatus request;
            ns2__getStatusResponse response;
            CDBResponseHandler handler;
            const char *soap_action = NULL;

            if (proxy.getStatus(endpoint, soap_action, &request, response) == SOAP_OK) {
                if (NULL != response.return_) {
                    handler.analyze(response.return_->c_str()); // look for ok, warning, error. Last 2 throw an exception
                    result.assign(handler.getMessage());
                } else {
                    std::string nullResp("calibration.getStatus: CDB server returned no data (response std::string* is NULL)");
                    throw CDBPermanentException(nullResp);
                }

            } else {
                wrapCDBException(); // build an exception string and throw
            }

            return;

        }

        void Calibration::wrapCDBException() {

            proxy.soap_stream_fault(std::cerr);
            std::string pre("SOAP fault: ");
            std::string fault((char*) proxy.soap_fault_string());
            std::string detail((char*) proxy.soap_fault_detail());
            throw CDBException(pre.append(fault).append("\nDetail:").append(detail));
        }

        Calibration::~Calibration() {
            /* Delete data and free memory */
            proxy.destroy();
            std::cout << "Calibration proxy destroyed" << std::endl;
        }
    }
}

